Unit USTreeNode;

Interface

Uses
  Dialogs, USortable, UTreeNode;

Type
  TSTreeNode = Class(TTreeNode)
  Private
    Function MaxValue  : TSTreeNode;
    Function ReadValue : TSortable;

    Procedure DeleteRoot;

  Public
    Function Delete(PSortable : TSortable) : Boolean;
    Function Search(PSortable : TSortable) : TSTreeNode;

    Function IsBalanced : Boolean;

    Procedure Insert(PSortable : TSortable);

    Property Value : TSortable  Read ReadValue;
  End;

Implementation


////////////////////////////////////////////////////////////////////////////////
//  Function TSTreenode.MaxValue : TSTreeNode;                                //
//                                                                            //
//  Liefert den untergeordneten Knoten mit dem grten Wert zurck            //
////////////////////////////////////////////////////////////////////////////////

Function TSTreeNode.MaxValue;
Begin
  If RightNode.IsEmpty Then Begin
    Result := Self;
  End
  Else Begin
    Result := TSTreeNode(RightNode).MaxValue;
  End;
End;


////////////////////////////////////////////////////////////////////////////////
//  Procedure TSTreeNode.DeleteRoot;                                          //
//                                                                            //
//  Betrachtet sich selber als Wurzel und lscht sich unter                   //
//  Beibehaltung der Sortierung                                               //
////////////////////////////////////////////////////////////////////////////////

Procedure TSTreeNode.DeleteRoot;
Var
  Substitute : TSTreeNode;

  TmpLeft,
  TmpRight : TTreeNode;

Begin
  If Not Self.IsEmpty Then Begin
    If Self.IsLeaf Then Begin
      If LeftNode <> Nil Then
        LeftNode.Remove;

      If RightNode <> Nil Then
        RightNode.Remove;

      Self.SetNil;
    End
    Else If LeftNode.IsEmpty Then Begin
      LeftNode.Remove;

      TmpLeft  := RightNode.LeftNode;
      TmpRight := RightNode.RightNode;

      IValue := RightNode.Value;


// Fhrt aus unerfindlichen Grnden zu Fehlern /////////////////////////////////
//
//    LeftNode  := RightNode.LeftNode;
//    RightNode := RightNode.RightNode;
//
// Wenn auch prinzipiell richtig, fhrt dieses zu Exceptions. Daher ist es
// notwendig, TmpLeft und TmpRight zu verwenden.
////////////////////////////////////////////////////////////////////////////////


      LeftNode  := TmpLeft;
      RightNode := TmpRight;

      RightNode.Remove;
    End
    Else If RightNode.IsEmpty Then Begin
      RightNode.Remove;

      TmpLeft  := LeftNode.LeftNode;
      TmpRight := LeftNode.RightNode;

      IValue := LeftNode.Value;

      LeftNode  := TmpLeft;
      RightNode := TmpRight;

      LeftNode.Remove;
    End
    Else Begin
      Substitute := TSTreeNode(LeftNode).MaxValue;

      // Bei Substitute mu mindestens der rechte Teilbaum leer sein,
      // der linke Teilbaum kann jedoch auch leer sein.

      IValue := Substitute.Value;

      Substitute.DeleteRoot;
    End;
  End;
End;


////////////////////////////////////////////////////////////////////////////////
//  Function TSTreeNode.Search(PSortable : TSortable) : TSTreeNode;           //
//                                                                            //
//  Such nach dem Knoten mit dem bergebenen Wert PSortable und gibt ihn      //
//  zurck, falls er gefunden werden konnte                                   //
////////////////////////////////////////////////////////////////////////////////

Function TSTreeNode.Search;
Begin
  If IsEmpty Then Begin
    Result := Nil;
  End
  Else Begin
    If PSortable.Equal(TSortable(Value)) Then Begin
      Result := Self;
    End
    Else Begin
      If PSortable.LT(TSortable(Value)) Then Begin
        Result := TSTreeNode(LeftNode).Search(PSortable);
      End
      Else Begin
        Result := TSTreeNode(RightNode).Search(PSortable);
      End;
    End;
  End;
End;


////////////////////////////////////////////////////////////////////////////////
//  TSTreeNode.Insert(PSortable : TSortable);                                 //
//                                                                            //
//  Fgt einen den als PSortable bergebenen Wert in einen Knoten (Blatt)     //
//  der Struktur ein; nach dem Einfgen in das Blatt werden neue (leere)      //
//  Knoten dem Blatt unterstellt.                                             //
////////////////////////////////////////////////////////////////////////////////

Procedure TSTreeNode.Insert;
Begin
  If IsEmpty Then Begin
    IValue := PSortable;

    LeftNode  := TSTreeNode.Create;
    RightNode := TSTreeNode.Create;
  End
  Else If PSortable.LT(TSortable(Value)) Then Begin
    TSTreeNode(LeftNode).Insert(PSortable);
  End
  Else Begin
    TSTreeNode(RightNode).Insert(PSortable);
  End;
End;


////////////////////////////////////////////////////////////////////////////////
//  Function TSTreeNode.Delete(PSortable : TSortable) : Boolean;              //
//                                                                            //
//  berprft, ob ein Knoten mit dem Wert PSortable in dem Baum vorhanden     //
//  ist und lscht diesen danach mittels der Prozedur DeleteRoot.             //
//  Da die leeren Knoten zum Einfgen bentigt werden (Prozedur Insert),      //
//  drfen diese nicht gelscht werden. Als Funktionswert wird zurckge-      //
//  geben, ob der Knoten gefunden und gelscht werden konnte.                 //
////////////////////////////////////////////////////////////////////////////////

Function TSTreeNode.Delete;
Var
  Node : TSTreeNode;

Begin
  Result := False;

  Node := Search(PSortable);

  If (Node <> Nil) And (Not Node.IsEmpty) Then Begin
    Node.DeleteRoot;
    Result := True;
  End;
End;


Function TSTreeNode.ReadValue;
Begin
  Result := TSortable(IValue);
End;


Function TSTreeNode.IsBalanced;
Var
  LeftDepth,
  RightDepth : Word;

Begin
  If IsLeaf Then Begin
    Result := True;
  End
  Else Begin
    LeftDepth  := LeftNode.Depth;
    RightDepth := RightNode.Depth;

    If (Abs(LeftDepth - RightDepth) <= 1) And TSTreeNode(LeftNode).IsBalanced And TSTreeNode(RightNode).IsBalanced Then Begin
      Result := True;
    End
    Else Begin
      Result := False;
    End;
  End;

End;



End.

