Unit USortables;

////////////////////////////////////////////////////////////////////////////////
//  USortables                                                                //
//                                                                            //
//  Enthlt sortierbare Elemente, die von TSortable ableleitet sind           //
////////////////////////////////////////////////////////////////////////////////



Interface

Uses
  SysUtils, USortable;

Type
  TInteger = Class(TSortable)
  Private
    IValue : Integer;  // Internal Value

    Procedure SetValue(PValue : Integer);

    Function GetValue : Integer;

  Public
    Function Equal(Element : TSortable) : Boolean; Override;
    Function LT(Element : TSortable) : Boolean; Override;  // Less Than
    Function Text : String; Override;

    Property Value : Integer Read GetValue Write SetValue;
  End;  // TInteger



  TReal = Class(TSortable)
  Private
    IValue : Real;  // Internal Value

    Procedure SetValue(PValue : Real);

    Function GetValue : Real;

  Public
    Function Equal(Element : TSortable) : Boolean; Override;
    Function LT(Element : TSortable) : Boolean; Override;  // Less Than
    Function Text : String; Override;

    Property Value : Real Read GetValue Write SetValue;
  End;  // TReal



  TString = Class(TSortable)
  Private
    IValue : String;  // Internal Value

    Procedure SetValue(PValue : String);

    Function GetValue : String;

  Public
    Function Equal(Element : TSortable) : Boolean; Override;
    Function LT(Element : TSortable) : Boolean; Override;  // Less Than
    Function Text : String; Override;

    Property Value : String Read GetValue Write SetValue;
  End;  // TString



Implementation


(******************************************************************************)
(**                                                                          **)
(**  TInteger                                                                **)
(**                                                                          **)
(******************************************************************************)


////////////////////////////////////////////////////////////////////////////////
//  TInteger.Equal                                                            //
//                                                                            //
//  Gibt an, ob das Element gleich ist                                        //
////////////////////////////////////////////////////////////////////////////////

Function TInteger.Equal;
Begin
  Result := Value = TInteger(Element).Value;
End;  // TInteger.Equal


////////////////////////////////////////////////////////////////////////////////
//  TInteger.LT                                                               //
//                                                                            //
//  Gibt an, ob das Element kleiner ist                                       //
////////////////////////////////////////////////////////////////////////////////

Function TInteger.LT;
Begin
  Result := Value < TInteger(Element).Value;
End;  // TInteger.LT


////////////////////////////////////////////////////////////////////////////////
//  TInteger.SetValue                                                         //
////////////////////////////////////////////////////////////////////////////////

Procedure TInteger.SetValue;
Begin
  IValue := PValue;
End;  // TInteger.SetValue


////////////////////////////////////////////////////////////////////////////////
//  TInteger.GetValue                                                         //
////////////////////////////////////////////////////////////////////////////////

Function TInteger.GetValue;
Begin
  Result := IValue;
End;  // TInteger.GetValue


////////////////////////////////////////////////////////////////////////////////
//  TInteger.Text                                                             //
////////////////////////////////////////////////////////////////////////////////

Function TInteger.Text;
Begin
  Result := IntToStr(IValue);
End;  // TInteger.Text



(******************************************************************************)
(**                                                                          **)
(**  TReal                                                                   **)
(**                                                                          **)
(******************************************************************************)


////////////////////////////////////////////////////////////////////////////////
//  TReal.Equal                                                               //
//                                                                            //
//  Gibt an, ob das Element gleich ist                                        //
////////////////////////////////////////////////////////////////////////////////

Function TReal.Equal;
Begin
  Result := Value = TReal(Element).Value;
End;  // TReal.Equal


////////////////////////////////////////////////////////////////////////////////
//  TReal.LT                                                                  //
//                                                                            //
//  Gibt an, ob das Element kleiner ist                                       //
////////////////////////////////////////////////////////////////////////////////

Function TReal.LT;
Begin
  Result := Value < TReal(Element).Value;
End;  // TReal.LT


////////////////////////////////////////////////////////////////////////////////
//  TReal.SetValue                                                            //
////////////////////////////////////////////////////////////////////////////////

Procedure TReal.SetValue;
Begin
  IValue := PValue;
End;  // TReal.SetValue


////////////////////////////////////////////////////////////////////////////////
//  TReal.GetValue                                                            //
////////////////////////////////////////////////////////////////////////////////

Function TReal.GetValue;
Begin
  Result := IValue;
End;  // TReal.GetValue


////////////////////////////////////////////////////////////////////////////////
//  TReal.Text                                                                //
////////////////////////////////////////////////////////////////////////////////

Function TReal.Text;
Begin
  Result := FloatToStr(IValue);
End;  // TReal.Text



(******************************************************************************)
(**                                                                          **)
(**  TString                                                                 **)
(**                                                                          **)
(******************************************************************************)


////////////////////////////////////////////////////////////////////////////////
//  TString.Equal                                                             //
//                                                                            //
//  Gibt an, ob das Element gleich ist                                        //
////////////////////////////////////////////////////////////////////////////////

Function TString.Equal;
Begin
  Result := Value = TString(Element).Value;
End;  // TString.Equal


////////////////////////////////////////////////////////////////////////////////
//  TString.LT                                                                //
//                                                                            //
//  Gibt an, ob das Element kleiner ist                                       //
////////////////////////////////////////////////////////////////////////////////

Function TString.LT;
Begin
  Result := Value < TString(Element).Value;
End;  // TString.LT


////////////////////////////////////////////////////////////////////////////////
//  TString.SetValue                                                          //
////////////////////////////////////////////////////////////////////////////////

Procedure TString.SetValue;
Begin
  IValue := PValue;
End;  // TString.SetValue


////////////////////////////////////////////////////////////////////////////////
//  TString.GetValue                                                          //
////////////////////////////////////////////////////////////////////////////////

Function TString.GetValue;
Begin
  Result := IValue;
End;  // TString.GetValue


////////////////////////////////////////////////////////////////////////////////
//  TString.Text                                                              //
////////////////////////////////////////////////////////////////////////////////

Function TString.Text;
Begin
  Result := IValue;
End;  // TString.Text


End.  // USortables

