Unit UTreeNode;

Interface


Type
  TTreeNode = Class(TObject)
  Private
    ILeftNode  : TTreeNode;
    IRightNode : TTreeNode;

    Function ReadValue     : TObject;
    Function ReadLeftNode  : TTreeNode;
    Function ReadRightNode : TTreeNode;

    Procedure WriteValue    (PValue : TObject);
    Procedure WriteLeftNode (PNode  : TTreeNode);
    Procedure WriteRightNode(PNode  : TTreeNode);

  Protected
    IValue : TObject;

  Public
    Procedure Remove;

    Procedure SetNil;

    Function Depth     : Word;
    Function IsLeaf    : Boolean;
    Function IsEmpty   : Boolean;
    Function OutDegree : Byte;

    Property Value     : TObject   Read ReadValue     Write WriteValue;
    Property LeftNode  : TTreeNode Read ReadLeftNode  Write WriteLeftNode;
    Property RightNode : TTreeNode Read ReadRightNode Write WriteRightNode;
  End;


Implementation


////////////////////////////////////////////////////////////////////////////////
//  Function TTreeNode.ReadValue;                                             //
//                                                                            //
//  Gibt IValue zurck.                                                       //
////////////////////////////////////////////////////////////////////////////////

Function TTreeNode.ReadValue;
Begin
  Result := IValue;
End;


////////////////////////////////////////////////////////////////////////////////
//  Function TTreeNode.ReadLeftNode;                                          //
//                                                                            //
//  Liefert ILeftNode zurck.                                                 //
////////////////////////////////////////////////////////////////////////////////

Function TTreeNode.ReadLeftNode;
Begin
  Result := ILeftNode;
End;


////////////////////////////////////////////////////////////////////////////////
//  Function TTreeNode.ReadRightNode;                                         //
//                                                                            //
//  Liefert IRightNode zurck.                                                //
////////////////////////////////////////////////////////////////////////////////

Function TTreeNode.ReadRightNode;
Begin
  Result := IRightNode;
End;


////////////////////////////////////////////////////////////////////////////////
//  Procedure TTreeNode.WriteValue(PValue : TObject);                         //
//                                                                            //
//  Weist IValue den Wert PValue zu.                                          //
////////////////////////////////////////////////////////////////////////////////

Procedure TTreeNode.WriteValue;
Begin
  IValue := PValue;
End;


////////////////////////////////////////////////////////////////////////////////
//  Procedure TTreeNode.WriteLeftNode(PNode : TTreeNode);                     //
//                                                                            //
//  Weist ILeftNode den Knoten PNode zu.                                      //
////////////////////////////////////////////////////////////////////////////////

Procedure TTreeNode.WriteLeftNode;
Begin
  ILeftNode := PNode;
End;


////////////////////////////////////////////////////////////////////////////////
//  Procedure TTreeNode.WriteRightNode(PNode : TTreeNode);                    //
//                                                                            //
//  Weist IRightNode den Knoten PNode zu.                                     //
////////////////////////////////////////////////////////////////////////////////

Procedure TTreeNode.WriteRightNode;
Begin
  IRightNode := PNode;
End;


////////////////////////////////////////////////////////////////////////////////
//  Procedure TTreeNode.SetNil;                                               //
//                                                                            //
//  Setzt alle internen Pointer auf Nil                                       //
////////////////////////////////////////////////////////////////////////////////

Procedure TTreeNode.SetNil;
Begin
  Value     := Nil;
  LeftNode  := Nil;
  RightNode := Nil;
End;


////////////////////////////////////////////////////////////////////////////////
//  Function TTreeNode.OutDegree : Byte;                                      //
//                                                                            //
//  Liefert den OutDegree des Knotens zurck.                                 //
////////////////////////////////////////////////////////////////////////////////

Function TTreeNode.OutDegree;
Var
  Counter : Byte;

Begin
  Counter := 0;

  If LeftNode <> Nil Then
    If Not LeftNode.IsEmpty Then
      Inc(Counter);

  If RightNode <> Nil Then
    If Not RightNode.IsEmpty Then
      Inc(Counter);

  Result := Counter;
End;


////////////////////////////////////////////////////////////////////////////////
//  Function TTreeNode.IsLeaf : Boolean;                                      //
//                                                                            //
//  berprft, ob der Knoten ein Blatt ist.                                   //
////////////////////////////////////////////////////////////////////////////////

Function TTreeNode.IsLeaf;
Begin
  Result := (OutDegree = 0);
End;


////////////////////////////////////////////////////////////////////////////////
//  Function TTreeNode.IsEmpty;                                               //
//                                                                            //
//  Gibt an, ob der Knoten einen Wert (Value) zugewiesen bekommen hat.        //
////////////////////////////////////////////////////////////////////////////////

Function TTreeNode.IsEmpty;
Begin
  Result := (IValue = Nil);
End;


////////////////////////////////////////////////////////////////////////////////
//  Function TTreeNode.Depth;                                                 //
//                                                                            //
//  Gibt die Tiefe des Knotens an.                                            //
////////////////////////////////////////////////////////////////////////////////

Function TTreeNode.Depth;
Begin
  If Self.IsEmpty Then Begin
    Result := 0;
  End
  Else Begin
    If LeftNode.IsEmpty And RightNode.IsEmpty Then Begin
      Result := 1;
    End
    Else Begin
      If LeftNode.Depth > RightNode.Depth Then Begin
        Result := LeftNode.Depth + 1;
      End
      Else Begin
        Result := RightNode.Depth + 1;
      End;
    End;
  End;
End;


Procedure TTreeNode.Remove;
Begin
  If LeftNode <> Nil Then
    LeftNode.Remove;

  If RightNode <> Nil Then
    RightNode.Remove;

  Inherited Destroy;
End;


End.
